@@ -1,6 +1,6 @@
-from __future__ import unicode_literals
-import sys
 import operator
+import sys
+
 
 try:
     from collections import MutableMapping, Sequence  # noqa
@@ -8,9 +8,12 @@ except ImportError:
     from collections.abc import MutableMapping, Sequence  # noqa
 
 PY3 = sys.version_info[0] >= 3
+PY26 = sys.version_info[:2] == (2, 6)
 
 if PY3:
     zip = zip
+    from functools import lru_cache
+    from io import StringIO
     from urllib.parse import (
         unquote, urljoin, urlunsplit, SplitResult, urlsplit as _urlsplit
     )
@@ -20,6 +23,7 @@ if PY3:
     iteritems = operator.methodcaller("items")
 else:
     from itertools import izip as zip  # noqa
+    from StringIO import StringIO
     from urlparse import (
         urljoin, urlunsplit, SplitResult, urlsplit as _urlsplit # noqa
     )
@@ -29,6 +33,11 @@ else:
     int_types = int, long
     iteritems = operator.methodcaller("iteritems")
 
+    if PY26:
+        from repoze.lru import lru_cache
+    else:
+        from functools32 import lru_cache
+
 
 # On python < 3.3 fragments are not handled properly with unknown schemes
 def urlsplit(url):
